@@ -32,6 +32,7 @@ from discord_typings.interactions.receiving import (
 )
 
 import naff.api.events as events
+import naff.client.const as constants
 from naff.api.events import RawGatewayEvent, MessageCreate
 from naff.api.events import processors
 from naff.api.events.internal import Component, BaseEvent
@@ -39,7 +40,7 @@ from naff.api.gateway.gateway import GatewayClient
 from naff.api.gateway.state import ConnectionState
 from naff.api.http.http_client import HTTPClient
 from naff.client import errors
-from naff.client.const import logger_name, GLOBAL_SCOPE, MISSING, MENTION_PREFIX, Absent, EMBED_MAX_DESC_LENGTH
+from naff.client.const import GLOBAL_SCOPE, MISSING, MENTION_PREFIX, Absent, EMBED_MAX_DESC_LENGTH, logger
 from naff.client.errors import (
     BotException,
     ExtensionLoadException,
@@ -51,7 +52,7 @@ from naff.client.errors import (
 )
 from naff.client.smart_cache import GlobalCache
 from naff.client.utils.input_utils import get_first_word, get_args
-from naff.client.utils.misc_utils import wrap_partial, get_event_name
+from naff.client.utils.misc_utils import get_event_name, wrap_partial
 from naff.client.utils.serializer import to_image_data
 from naff.models import (
     Activity,
@@ -102,12 +103,12 @@ from naff.models.naff.tasks import Task
 if TYPE_CHECKING:
     from naff.models import Snowflake_Type, TYPE_ALL_CHANNEL
 
-log = logging.getLogger(logger_name)
 
 __all__ = ("Client",)
 
 
 class Client(
+    processors.AutoModEvents,
     processors.ChannelEvents,
     processors.GuildEvents,
     processors.MemberEvents,
@@ -154,6 +155,9 @@ class Client(
 
         debug_scope: Snowflake_Type: Force all application commands to be registered within this scope
         asyncio_debug: bool: Enable asyncio debug features
+        basic_logging: bool: Utilise basic logging to output library data to console. Do not use in combination with `Client.logger`
+        logging_level: int: The level of logging to use for basic_logging. Do not use in combination with `Client.logger`
+        logger: logging.Logger: The logger NAFF should use. Do not use in combination with `Client.basic_logging` and `Client.logging_level`. Note: Different loggers with multiple clients are not supported
 
     Optionally, you can configure the caches here, by specifying the name of the cache, followed by a dict-style object to use.
     It is recommended to use `smart_cache.create_cache` to configure the cache here.
@@ -181,6 +185,7 @@ class Client(
         global_pre_run_callback: Absent[Callable[..., Coroutine]] = MISSING,
         intents: Union[int, Intents] = Intents.DEFAULT,
         interaction_context: Type[InteractionContext] = InteractionContext,
+        logger: logging.Logger = logger,
         modal_context: Type[ModalContext] = ModalContext,
         prefixed_context: Type[PrefixedContext] = PrefixedContext,
         send_command_tracebacks: bool = True,
@@ -189,11 +194,20 @@ class Client(
         sync_interactions: bool = True,
         sync_ext: bool = True,
         total_shards: int = 1,
+        basic_logging: bool = False,
+        logging_level: int = logging.INFO,
         **kwargs,
     ) -> None:
+        if basic_logging:
+            logging.basicConfig()
+            logger.setLevel(logging_level)
+
+        # Set Up logger and overwrite the constant
+        self.logger = logger
+        """The logger NAFF should use. Do not use in combination with `Client.basic_logging` and `Client.logging_level`. Note: Different loggers with multiple clients are not supported"""
+        constants.logger = logger
 
         # Configuration
-
         self.sync_interactions = sync_interactions
         """Should application commands be synced"""
         self.del_unused_app_cmd: bool = delete_unused_application_cmds
@@ -214,6 +228,7 @@ class Client(
             auto_defer = auto_defer or AutoDefer()
         self.auto_defer = auto_defer
         """A system to automatically defer commands after a set duration"""
+        self.intents = intents
 
         # resources
 
@@ -332,11 +347,6 @@ class Client(
         """Returns if the gateway has been started."""
         return self._connection_state.gateway_started.is_set()
 
-    @property
-    def intents(self) -> Intents:
-        """The intents being used by this bot."""
-        return self._connection_state.intents
-
     @property
     def user(self) -> NaffUser:
         """Returns the bot's user."""
@@ -389,9 +399,12 @@ class Client(
         """Returns the websocket client."""
         return self._connection_state.gateway
 
+    def get_guild_websocket(self, id: "Snowflake_Type") -> GatewayClient:
+        return self.ws
+
     def _sanity_check(self) -> None:
         """Checks for possible and common errors in the bot's configuration."""
-        log.debug("Running client sanity checks...")
+        logger.debug("Running client sanity checks...")
         contexts = {
             self.interaction_context: InteractionContext,
             self.prefixed_context: PrefixedContext,
@@ -404,20 +417,20 @@ class Client(
                 raise TypeError(f"{obj.__name__} must inherit from {expected.__name__}")
 
         if self.del_unused_app_cmd:
-            log.warning(
+            logger.warning(
                 "As `delete_unused_application_cmds` is enabled, the client must cache all guilds app-commands, this could take a while."
             )
 
         if Intents.GUILDS not in self._connection_state.intents:
-            log.warning("GUILD intent has not been enabled; this is very likely to cause errors")
+            logger.warning("GUILD intent has not been enabled; this is very likely to cause errors")
 
         if self.fetch_members and Intents.GUILD_MEMBERS not in self._connection_state.intents:
             raise BotException("Members Intent must be enabled in order to use fetch members")
         elif self.fetch_members:
-            log.warning("fetch_members enabled; startup will be delayed")
+            logger.warning("fetch_members enabled; startup will be delayed")
 
         if len(self.processors) == 0:
-            log.warning("No Processors are loaded! This means no events will be processed!")
+            logger.warning("No Processors are loaded! This means no events will be processed!")
 
     async def generate_prefixes(self, bot: "Client", message: Message) -> str | Iterable[str]:
         """
@@ -473,7 +486,7 @@ class Client(
             except Exception:  # noqa : S110
                 pass
 
-        log.error(
+        logger.error(
             "Ignoring exception in {}:{}{}".format(source, "\n" if len(out) > 1 else " ", "".join(out)),
         )
 
@@ -551,7 +564,7 @@ class Client(
             symbol = "/"
         else:
             symbol = "?"  # likely custom context
-        log.info(f"Command Called: {symbol}{ctx.invoke_target} with {ctx.args = } | {ctx.kwargs = }")
+        logger.info(f"Command Called: {symbol}{ctx.invoke_target} with {ctx.args = } | {ctx.kwargs = }")
 
     async def on_component_error(self, ctx: ComponentContext, error: Exception, *args, **kwargs) -> None:
         """
@@ -575,7 +588,7 @@ class Client(
 
         """
         symbol = "¢"
-        log.info(f"Component Called: {symbol}{ctx.invoke_target} with {ctx.args = } | {ctx.kwargs = }")
+        logger.info(f"Component Called: {symbol}{ctx.invoke_target} with {ctx.args = } | {ctx.kwargs = }")
 
     async def on_autocomplete_error(self, ctx: AutocompleteContext, error: Exception, *args, **kwargs) -> None:
         """
@@ -604,7 +617,7 @@ class Client(
 
         """
         symbol = "$"
-        log.info(f"Autocomplete Called: {symbol}{ctx.invoke_target} with {ctx.args = } | {ctx.kwargs = }")
+        logger.info(f"Autocomplete Called: {symbol}{ctx.invoke_target} with {ctx.args = } | {ctx.kwargs = }")
 
     @Listener.create()
     async def on_resume(self) -> None:
@@ -630,11 +643,11 @@ class Client(
                     # ensure all guilds have completed chunking
                     for guild in self.guilds:
                         if guild and not guild.chunked.is_set():
-                            log.debug(f"Waiting for {guild.id} to chunk")
+                            logger.debug(f"Waiting for {guild.id} to chunk")
                             await guild.chunked.wait()
 
             except asyncio.TimeoutError:
-                log.warning("Timeout waiting for guilds cache: Not all guilds will be in cache")
+                logger.warning("Timeout waiting for guilds cache: Not all guilds will be in cache")
                 break
             self._guild_event.clear()
 
@@ -642,6 +655,13 @@ class Client(
                 # all guilds cached
                 break
 
+        if self.fetch_members:
+            # ensure all guilds have completed chunking
+            for guild in self.guilds:
+                if guild and not guild.chunked.is_set():
+                    logger.debug(f"Waiting for {guild.id} to chunk")
+                    await guild.chunked.wait()
+
         # run any pending startup tasks
         if self.async_startup_tasks:
             try:
@@ -675,7 +695,7 @@ class Client(
         # so im gathering commands here
         self._gather_commands()
 
-        log.debug("Attempting to login")
+        logger.debug("Attempting to login")
         me = await self.http.login(token.strip())
         self._user = NaffUser.from_dict(me, self)
         self.cache.place_user_data(me)
@@ -726,7 +746,7 @@ class Client(
 
     async def stop(self) -> None:
         """Shutdown the bot."""
-        log.debug("Stopping the bot.")
+        logger.debug("Stopping the bot.")
         self._ready.clear()
         await self.http.close()
         await self._connection_state.stop()
@@ -741,7 +761,7 @@ class Client(
         """
         listeners = self.listeners.get(event.resolved_name, [])
         if listeners:
-            log.debug(f"Dispatching Event: {event.resolved_name}")
+            logger.debug(f"Dispatching Event: {event.resolved_name}")
             event.bot = self
             for _listen in listeners:
                 try:
@@ -1030,13 +1050,17 @@ class Client(
                 elif isinstance(func, Listener):
                     self.add_listener(func)
 
-            log.debug(f"{len(_cmds)} commands have been loaded from `__main__` and `client`")
+            logger.debug(f"{len(_cmds)} commands have been loaded from `__main__` and `client`")
 
         process(
             [obj for _, obj in inspect.getmembers(sys.modules["__main__"]) if isinstance(obj, (BaseCommand, Listener))]
         )
         process(
-            [wrap_partial(obj, self) for _, obj in inspect.getmembers(self) if isinstance(obj, (BaseCommand, Listener))]
+            [
+                obj.copy_with_binding(self)
+                for _, obj in inspect.getmembers(self)
+                if isinstance(obj, (BaseCommand, Listener))
+            ]
         )
 
         [wrap_partial(obj, self) for _, obj in inspect.getmembers(self) if isinstance(obj, Task)]
@@ -1083,7 +1107,7 @@ class Client(
 
         for scope, remote_cmds in results.items():
             if remote_cmds == MISSING:
-                log.debug(f"Bot was not invited to guild {scope} with `application.commands` scope")
+                logger.debug(f"Bot was not invited to guild {scope} with `application.commands` scope")
                 continue
 
             remote_cmds = {cmd_data["name"]: cmd_data for cmd_data in remote_cmds}
@@ -1095,7 +1119,7 @@ class Client(
                     if cmd_data is MISSING:
                         if cmd_name not in found:
                             if warn_missing:
-                                log.error(
+                                logger.error(
                                     f'Detected yet to sync slash command "/{cmd_name}" for scope '
                                     f"{'global' if scope == GLOBAL_SCOPE else scope}"
                                 )
@@ -1107,17 +1131,28 @@ class Client(
 
             if warn_missing:
                 for cmd_data in remote_cmds.values():
-                    log.error(
+                    logger.error(
                         f"Detected unimplemented slash command \"/{cmd_data['name']}\" for scope "
                         f"{'global' if scope == GLOBAL_SCOPE else scope}"
                     )
 
-    async def synchronise_interactions(self) -> None:
-        """Synchronise registered interactions with discord."""
+    async def synchronise_interactions(
+        self, *, scopes: list["Snowflake_Type"] = MISSING, delete_commands: Absent[bool] = MISSING
+    ) -> None:
+        """
+        Synchronise registered interactions with discord.
+
+        Args:
+            scopes: Optionally specify which scopes are to be synced
+            delete_commands: Override the client setting and delete commands
+        """
         s = time.perf_counter()
+        _delete_cmds = self.del_unused_app_cmd if delete_commands is MISSING else delete_commands
         await self._cache_interactions()
 
-        if self.del_unused_app_cmd:
+        if scopes is not MISSING:
+            cmd_scopes = scopes
+        elif self.del_unused_app_cmd:
             # if we're deleting unused commands, we check all scopes
             cmd_scopes = [to_snowflake(g_id) for g_id in self._user._guild_ids] + [GLOBAL_SCOPE]
         else:
@@ -1135,7 +1170,7 @@ class Client(
                 try:
                     remote_commands = await self.http.get_application_commands(self.app.id, cmd_scope)
                 except Forbidden:
-                    log.warning(f"Bot is lacking `application.commands` scope in {cmd_scope}!")
+                    logger.warning(f"Bot is lacking `application.commands` scope in {cmd_scope}!")
                     return
 
                 for local_cmd in self.interactions.get(cmd_scope, {}).values():
@@ -1153,25 +1188,25 @@ class Client(
                         # determine if the local and remote commands are out-of-sync
                         sync_needed_flag = True
                         sync_payload.append(local_cmd_json)
-                    elif not self.del_unused_app_cmd and remote_cmd_json:
+                    elif not _delete_cmds and remote_cmd_json:
                         _remote_payload = {
                             k: v for k, v in remote_cmd_json.items() if k not in ("id", "application_id", "version")
                         }
                         sync_payload.append(_remote_payload)
-                    elif self.del_unused_app_cmd:
+                    elif _delete_cmds:
                         sync_payload.append(local_cmd_json)
 
                 sync_payload = [json.loads(_dump) for _dump in {json.dumps(_cmd) for _cmd in sync_payload}]
 
-                if sync_needed_flag or (self.del_unused_app_cmd and len(sync_payload) < len(remote_commands)):
+                if sync_needed_flag or (_delete_cmds and len(sync_payload) < len(remote_commands)):
                     # synchronise commands if flag is set, or commands are to be deleted
-                    log.info(f"Overwriting {cmd_scope} with {len(sync_payload)} application commands")
+                    logger.info(f"Overwriting {cmd_scope} with {len(sync_payload)} application commands")
                     sync_response: list[dict] = await self.http.overwrite_application_commands(
                         self.app.id, sync_payload, cmd_scope
                     )
                     self._cache_sync_response(sync_response, cmd_scope)
                 else:
-                    log.debug(f"{cmd_scope} is already up-to-date with {len(remote_commands)} commands.")
+                    logger.debug(f"{cmd_scope} is already up-to-date with {len(remote_commands)} commands.")
 
             except Forbidden as e:
                 raise InteractionMissingAccess(cmd_scope) from e
@@ -1181,7 +1216,7 @@ class Client(
         await asyncio.gather(*[sync_scope(scope) for scope in cmd_scopes])
 
         t = time.perf_counter() - s
-        log.debug(f"Sync of {len(cmd_scopes)} scopes took {t} seconds")
+        logger.debug(f"Sync of {len(cmd_scopes)} scopes took {t} seconds")
 
     def get_application_cmd_by_id(self, cmd_id: "Snowflake_Type") -> Optional[InteractionCommand]:
         """
@@ -1211,9 +1246,9 @@ class Client(
                     output = e.search_for_message(e.errors[cmd_num], cmd)
                     if len(output) > 1:
                         output = "\n".join(output)
-                        log.error(f"Multiple Errors found in command `{cmd['name']}`:\n{output}")
+                        logger.error(f"Multiple Errors found in command `{cmd['name']}`:\n{output}")
                     else:
-                        log.error(f"Error in command `{cmd['name']}`: {output[0]}")
+                        logger.error(f"Error in command `{cmd['name']}`: {output[0]}")
             else:
                 raise e from None
         except Exception:
@@ -1351,7 +1386,7 @@ class Client(
                 ctx = await self.get_context(interaction_data, True)
 
                 ctx.command: SlashCommand = self.interactions[scope][ctx.invoke_target]  # type: ignore
-                log.debug(f"{scope} :: {ctx.command.name} should be called")
+                logger.debug(f"{scope} :: {ctx.command.name} should be called")
 
                 if ctx.command.auto_defer:
                     auto_defer = ctx.command.auto_defer
@@ -1380,7 +1415,7 @@ class Client(
                     finally:
                         await self.on_command(ctx)
             else:
-                log.error(f"Unknown cmd_id received:: {interaction_id} ({name})")
+                logger.error(f"Unknown cmd_id received:: {interaction_id} ({name})")
 
         elif interaction_data["type"] == InteractionTypes.MESSAGE_COMPONENT:
             # Buttons, Selects, ContextMenu::Message
@@ -1576,7 +1611,7 @@ class Client(
             raise ExtensionLoadException(f"Unexpected Error loading {name}") from e
 
         else:
-            log.debug(f"Loaded Extension: {name}")
+            logger.debug(f"Loaded Extension: {name}")
             self.__modules[name] = module
 
             if self.sync_ext and self._ready.is_set():
@@ -1639,7 +1674,7 @@ class Client(
         module = self.__modules.get(name)
 
         if module is None:
-            log.warning("Attempted to reload extension thats not loaded. Loading extension instead")
+            logger.warning("Attempted to reload extension thats not loaded. Loading extension instead")
             return self.load_extension(name, package)
 
         if not load_kwargs:
